<?php
/**
 * Set the site user visiting.
 *
 * @access public
 * @return void
 */
public function setSite()
{
    $this->config->site->locked = false;
    $openedDays = 0;
    if(isset($this->config->global->openedDate)) $openedDays = (time() - strtotime($this->config->global->openedDate)) / 24 / 3600;

    /* Check site status. */
    //if(isset($this->config->global->siteStatus) and $this->config->global->siteStatus == 'try' and $openedDays > 3 ) $this->config->site->locked = true;
    if(isset($this->config->global->siteStatus) and $this->config->global->siteStatus == 'locked') $this->config->site->locked = true;

    if(!isset($this->config->site))                $this->config->site                = new stdclass();
    if(!isset($this->config->site->name))          $this->config->site->name          = $this->lang->chanzhiEPS;
    if(!isset($this->config->site->keywords))      $this->config->site->keywords      = '';
    if(!isset($this->config->site->indexKeywords)) $this->config->site->indexKeywords = '';
    if(!isset($this->config->site->slogan))        $this->config->site->slogan        = '';
    if(!isset($this->config->site->copyright))     $this->config->site->copyright     = '';
    if(!isset($this->config->site->icpSN))         $this->config->site->icpSN         = '';
    if(!isset($this->config->site->meta))          $this->config->site->meta          = '';
    if(!isset($this->config->site->desc))          $this->config->site->desc          = '';
    if(!isset($this->config->site->theme))         $this->config->site->theme         = 'default';
    if(!isset($this->config->site->lang))          $this->config->site->lang          = 'zh-cn';
    if(!isset($this->config->site->menu))          $this->config->site->menu          = json_encode(array());

    if(!isset($this->config->company))             $this->config->company             = new stdclass();
    if(!isset($this->config->company->name))       $this->config->company->name       = '';
    if(!isset($this->config->company->desc))       $this->config->company->desc       = '';
    if(!isset($this->config->company->content))    $this->config->company->content    = '';
    if(!isset($this->config->company->contact))    $this->config->company->contact    = json_encode(array());

    if(!isset($this->config->product))             $this->config->product             = new stdclass();
    if(isset($this->config->global->theme2Import) and strpos($this->config->global->theme2Import, 'qd0') === 0)   
    {
        $this->loadModel('install')->importTheme();
    }
}

/**
 * Get backup 
 * 
 * @access public
 * @return bool|object
 */
public function getBackup()
{
    $domain = $this->config->site->code;
    $domain = strtolower($domain);

    $siteBackupRoot = dirname($this->app->getAppRoot()) . '/backup/' . $domain . '/';
    if(!file_exists($siteBackupRoot)) return false;
    $backup = new stdclass();
    $backup->sql = new stdclass();
    $backup->sql->fileName = $domain . '.sql.zip';
    if(!file_exists($siteBackupRoot . $backup->sql->fileName)) $backup->sql->fileName = $domain . '.sql.bz2';
    $backup->sql->fileSize = round(filesize($siteBackupRoot . $backup->sql->fileName) / 1024, 2);
    $backup->sql->fileTime = date('n/d H:i', filectime($siteBackupRoot . $backup->sql->fileName));

    $backup->file = new stdclass();
    $backup->file->fileName = $domain . '.file.zip';
    if(!file_exists($siteBackupRoot . $backup->file->fileName)) $backup->file->fileName = $domain . '.file.bz2';
    $backup->file->fileSize = file_exists($siteBackupRoot . $backup->file->fileName) ? round(filesize($siteBackupRoot . $backup->file->fileName) / 1024, 2) : 0;
    $backup->file->fileTime = file_exists($siteBackupRoot . $backup->file->fileName) ? date('n/d H:i', filectime($siteBackupRoot . $backup->file->fileName)) : '0/0 00:00';

    return $backup;
}

/**
 * getTrafficAndUsage 
 * 
 * @param  string $siteID 
 * @access public
 * @return void
 */
public function getTrafficAndUsage($siteID)
{
    $database = $this->loadModel('install')->getNameOfDB($siteID);
    $re = $this->dbh->query("USE $database");

    $result = $this->dao->select('`key`,`value`')->from($database . '.' . TABLE_CONFIG)
        ->where('`owner`')->eq('system')
        ->andWhere('`module`')->eq('common')
        ->andWhere('`section`')->eq('global')
        ->fetchAll('key');

    $firstDay = date('Y-m') . '-01';
    $days     = $this->dao->select("cdn, master, total, date")->from('eps_traffic')->where('date')->ge($firstDay)->fetchAll('date');

    $data = new stdclass();
    $data->usage         = isset($result['usage']) ? $result['usage']->value : 0;
    $data->traffic       = isset($result['traffic']) ? $result['traffic']->value : 0;
    $data->cdnTraffic    = isset($result['cdnTraffic']) ? $result['cdnTraffic']->value : 0;
    $data->masterTraffic = isset($result['masterTraffic']) ? $result['masterTraffic']->value : 0;
    $data->days          = $days;
    return $data;
}

public function saveVip($domain)
{
    $vipConfigFile = dirname($this->app->getBasePath()) . DS . 'config/vip';
    return file_put_contents($vipConfigFile, $domain . ',', FILE_APPEND);
}

/**
 * Create a backup.
 * 
 * @param  string    $password 
 * @access public
 * @return array
 */
public function createBackup($password)
{
    /* Start backup db.*/
    $this->backupDB($password);

    /* Backup cdn function. */
    $site = $this->config->site->code;

    $today       = date('ymd');
    $firstChar   = $site{0};
    $sitePath    = substr($site, 0, 1) . '/' . $site;
    $attatchPath = "/home/z/node/www/data/upload/{$sitePath}";
    $slidesPath  = "/home/z/node/www/data/slides/{$sitePath}";
    $sourcePath  = "/home/z/node/www/data/source/{$sitePath}";
    $backupPath  = "/home/z/backup/data/" . $sitePath;
    $fileName    = substr(md5(uniqid($site)), 1, rand(7, 10)) . date('ymd');
    $tmpPath     = "/tmp/chanzhiAttatch/{$sitePath}";

    $commads  = array();

    $commads[] = "mkdir -p $backupPath";
    $commads[] = "touch {$backupPath}/{$fileName}.zip";
    $commads[] = "rm -f {$backupPath}/{$fileName}.zip";

    $commads[] = "mkdir -p $attatchPath";
    $commads[] = "touch $attatchPath/readme.txt";

    $commads[] = "mkdir -p $slidesPath";
    $commads[] = "touch $slidesPath/readme.txt";

    $commads[] = "mkdir -p $sourcePath";
    $commads[] = "touch $sourcePath/readme.txt";

    $commads[] = "mkdir -p $tmpPath";
    $commads[] = "mkdir -p {$tmpPath}/upload/{$firstChar}";
    $commads[] = "mkdir -p {$tmpPath}/slides/{$firstChar}";
    $commads[] = "mkdir -p {$tmpPath}/source/{$firstChar}";
    $commads[] = "cp -rf {$attatchPath} {$tmpPath}/upload/{$firstChar}/";
    $commads[] = "cp -rf {$slidesPath}  {$tmpPath}/slides/{$firstChar}/";
    $commads[] = "cp -rf {$sourcePath}  {$tmpPath}/source/{$firstChar}/";
    $commads[] = "cd {$tmpPath}; cd ../; zip -r -P {$password} {$backupPath}/{$fileName}.zip {$site}";
    $commads[] = "htpasswd -D /home/z/backup/data/.userpass {$this->app->user->account}";
    $commads[] = "htpasswd -b /home/z/backup/data/.userpass {$this->app->user->account} {$password}";

    $usage       = zget($this->config->global, 'usage', 0);
    $canDownLoad = (($usage < 10 * 1024 *1024) or (date('H') > 18) or (date('H') < 5));
    if(!$canDownLoad) $commads[] = "chmod 000 {$backupPath}/{$fileName}.zip;";

    foreach($commads as $commad) `$commad`;
        
    return array('canDownLoad' => $canDownLoad, 'fileName' => $fileName);
}

/**
 * Backup database. 
 * 
 * @param  string    $password 
 * @access public
 * @return void
 */
public function backupDB($password)
{
    $code   = $this->config->site->code;
    $siteDB = "chanzhi_{$code}";

    $backDir = dirname($this->app->getBasePath()) . "/backup/{$code}/";
    $dbFile  = $backDir . $code . '.sql';

    $backCMD = '';
    if(!is_dir($backDir)) $backCMD .= "mkdir -p $backDir;\n";
    $backCMD .= "mysqldump -u root -pchanzhi $siteDB > $dbFile;\n";

    if(file_exists($dbFile . '.zip')) $backCMD .= "rm $dbFile.zip;\n";
    $backCMD .= "cd $backDir; zip -r -P $password {$dbFile}.zip *.sql;\n";
    $backCMD .= "rm -fr $dbFile;\n";
    `$backCMD`;
}

/**
 * Send backup email.
 * 
 * @param  string    $fileName 
 * @access public
 * @return void
 */
public function sendBackup($password, $fileName, $canDownLoad = false)
{
    $site = $this->config->site->code;
    $sitePath = substr($site, 0, 1) . '/' . $site . '/';
    $this->app->loadClass('zcrypt');

    $password = zcrypt::decrypt($this->config->global->backupKey, $this->config->chanzhi->cryptKey);

    $fileDownLink = $this->config->chanzhi->cdnBackupURL . '/' . $sitePath . $fileName . '.zip';
    $deletedTime  = date('Y-m-d', strtotime('+1 day'));

    if($canDownLoad)  $content = sprintf($this->lang->site->backup->backupInfo['downloadNow'], $fileDownLink, $fileDownLink, $deletedTime);
    if(!$canDownLoad) $content = sprintf($this->lang->site->backup->backupInfo['downloadTonight'], $fileDownLink, $fileDownLink, $deletedTime);
    $this->loadModel('mail')->send($this->app->user->account, $this->lang->site->backup->backupDownLink, $content, true); 
    return !$this->mail->isError(); 
}

public function syncSiteCodes()
{
    $http   = $this->app->loadClass('http');
    $result = $http->get($this->config->zhanqun->api->plat->host . "/admin.php?m=portal&f=apigetsitecode&t=json&key=" . $this->config->zhanqun->api->key);
    $result = json_decode($result);

    if(!isset($result->status)) return false;
    if($result->status != 'success') return false;
    if(isset($result->data) and md5($result->data) != $result->md5) return false;
    if(isset($result->data)) $data = json_decode($result->data, true);
    $domain2ID   = var_export($data['domain2ID'], true);
    $domain2Code = var_export($data['domain2Code'], true);

    $content = "<?php\n";
    $content .= '$config->site->domain2ID = ';
    $content .= "\n";
    $content .= $domain2ID;
    $content .= ";\n";
    $content .= '$config->site->domain2Code = ';
    $content .= "\n";
    $content .= $domain2Code;
    $content .= ";\n";
    $content .= '$config->site->code2Domain = ';
    $content .= 'array_flip($config->site->domain2Code)';
    $content .= ";\n";
    $content .= '$config->site->id2Domain = ';
    $content .= 'array_flip($config->site->domain2ID)';
    $content .= ";\n";
    $configFile = dirname($this->app->getBasePath()) . DS . 'system/config/ext/sitedomain.php';
    file_put_contents($configFile, $content);

    $proConfigFile = dirname($this->app->getBasePath()) . DS . 'prosystem/config/ext/sitedomain.php';
    file_put_contents($proConfigFile, $content);
}

public function setSystem($config = null)
{
    $errors ='';
    if(empty($config)) $config = fixer::input('post')->get();
    
    $myFile = $this->app->getConfigRoot() . 'sites/' . $this->config->site->code . '.php'; 
    if(!is_file($myFile))
    {
        touch($myFile);
        chmod($myFile, 0777);  
        file_put_contents($myFile, "<?php" . "\n");
    }
    
    $rawContent = ''; 
    if(is_file($myFile))
    {
        $rawContent = file_get_contents($myFile);
        $rawContent = preg_replace('/.*config\->cn2tw.*\n/', '', $rawContent);
        $rawContent = preg_replace('/.*config\->enabledLangs.*\n/', '', $rawContent);
        $rawContent = preg_replace('/.*config\->defaultLang.*\n/', '', $rawContent);
        $rawContent = str_replace("\n\n", "\n", $rawContent);
        $rawContent = str_replace('?>', '', $rawContent);
    }

    if(is_writable($myFile) !== true)
    {
        $error = sprintf($this->lang->site->fileAuthority, 'chmod o=rwx ' . $myFile);
        $errors['submit'] = $error;
        return array('result' => 'fail', 'message' => $errors);
    }        
    else
    {
        $content = '';

        if(isset($config->enabledLangs))
        {
            $enabledLangs = array();
            foreach($config->enabledLangs as $lang)
            {
                if(isset($this->config->langs[$lang])) $enabledLangs[] = $lang;
            }
            $content .= '$config->enabledLangs = \'' . join(',', $enabledLangs) . "';\n";
        }

        if(isset($config->cn2tw) and in_array('zh-tw', $enabledLangs))
        {
            if(is_array($config->cn2tw)) $config->cn2tw = $config->cn2tw[0];
            $config->cn2tw = (int) $config->cn2tw;
            $content .= '$config->cn2tw = ' . $config->cn2tw . ";\n";
        }

        if(isset($config->defaultLang) and in_array($config->defaultLang, $enabledLangs))
        {
            $content .= '$config->defaultLang = \'' . $config->defaultLang . "';\n";
        }

        file_put_contents($myFile, $rawContent . "\n" . $content);
        dao::$changedTables[] = TABLE_CONFIG;
        return array('result' => 'success', 'message' => $this->lang->saveSuccess); 
    }
}
